@@ -1,7 +1,7 @@
 package App::CSVUtils;
 
-our $DATE = '2016-11-04'; # DATE
-our $VERSION = '0.006'; # VERSION
+our $DATE = '2016-11-15'; # DATE
+our $VERSION = '0.007'; # VERSION
 
 use 5.010001;
 use strict;
@@ -126,7 +126,7 @@ my %arg_field_1 = (
     },
 );
 
-my %arg_field_nocomp = (
+my %arg_field_1_nocomp = (
     field => {
         summary => 'Field name',
         schema => 'str*',
@@ -175,6 +175,13 @@ my %args_sort_short = (
     },
 );
 
+my %arg_with_data_rows = (
+    with_data_rows => {
+        summary => 'Whether to also output data rows',
+        schema => 'bool',
+    },
+);
+
 $SPEC{csvutil} = {
     v => 1.1,
     summary => 'Perform action on a CSV file',
@@ -225,10 +232,13 @@ sub csvutil {
     my $code;
     my $field_idx;
     my $sorted_fields;
+    my @summary_row;
 
     while (my $row = $csv->getline($fh)) {
         $i++;
         if ($i == 1) {
+            # header row
+
             $fields = $row;
             for my $j (0..$#{$row}) {
                 unless (length $row->[$j]) {
@@ -258,7 +268,11 @@ sub csvutil {
                     if $args{sort_reverse};
                 $row = $sorted_fields;
             }
-        } # if i==1
+            if ($action eq 'sum' || $action eq 'avg') {
+                @summary_row = map {0} @$row;
+            }
+        } # if i==1 (header row)
+
         if ($action eq 'list-field-names') {
             return [200, "OK",
                     [map { {name=>$_, index=>$field_idxs{$_}+1} }
@@ -324,11 +338,46 @@ sub csvutil {
                 $row = \@new_row;
             }
             $res .= _get_csv_row($csv, $row, $i);
+        } elsif ($action eq 'sum') {
+            if ($i == 1) {
+                $res .= _get_csv_row($csv, $row, $i);
+            } else {
+                require Scalar::Util;
+                for (0..$#{$row}) {
+                    next unless Scalar::Util::looks_like_number($row->[$_]);
+                    $summary_row[$_] += $row->[$_];
+                }
+                $res .= _get_csv_row($csv, $row, $i)
+                    if $args{_with_data_rows};
+            }
+        } elsif ($action eq 'avg') {
+            if ($i == 1) {
+                $res .= _get_csv_row($csv, $row, $i);
+            } else {
+                require Scalar::Util;
+                for (0..$#{$row}) {
+                    next unless Scalar::Util::looks_like_number($row->[$_]);
+                    $summary_row[$_] += $row->[$_];
+                }
+                $res .= _get_csv_row($csv, $row, $i)
+                    if $args{_with_data_rows};
+            }
         } else {
             return [400, "Unknown action '$action'"];
         }
     }
 
+    if ($action eq 'sum') {
+        $res .= _get_csv_row($csv, \@summary_row,
+                             $args{_with_data_rows} ? $i+1 : 2);
+    } elsif ($action eq 'avg') {
+        if ($i > 2) {
+            for (@summary_row) { $_ /= ($i-1) }
+        }
+        $res .= _get_csv_row($csv, \@summary_row,
+                             $args{_with_data_rows} ? $i+1 : 2);
+    }
+
     [200, "OK", $res, {"cmdline.skip_format"=>1}];
 }
 
@@ -345,7 +394,7 @@ the first data row). Field will be added as the last field.
 _
     args => {
         %arg_filename_0,
-        %arg_field_1,
+        %arg_field_1_nocomp,
         %arg_eval_2,
     },
 };
@@ -473,6 +522,34 @@ sub csv_sort_fields {
     csvutil(%csvutil_args);
 }
 
+$SPEC{csv_sum} = {
+    v => 1.1,
+    summary => 'Output a summary row which are arithmetic sums of data rows',
+    args => {
+        %arg_filename_0,
+        %arg_with_data_rows,
+    },
+};
+sub csv_sum {
+    my %args = @_;
+
+    csvutil(%args, action=>'sum', _with_data_rows=>$args{with_data_rows});
+}
+
+$SPEC{csv_avg} = {
+    v => 1.1,
+    summary => 'Output a summary row which are arithmetic averages of data rows',
+    args => {
+        %arg_filename_0,
+        %arg_with_data_rows,
+    },
+};
+sub csv_avg {
+    my %args = @_;
+
+    csvutil(%args, action=>'avg', _with_data_rows=>$args{with_data_rows});
+}
+
 
 1;
 # ABSTRACT: CLI utilities related to CSV
@@ -489,7 +566,7 @@ App::CSVUtils - CLI utilities related to CSV
 
 =head1 VERSION
 
-This document describes version 0.006 of App::CSVUtils (from Perl distribution App-CSVUtils), released on 2016-11-04.
+This document describes version 0.007 of App::CSVUtils (from Perl distribution App-CSVUtils), released on 2016-11-15.
 
 =head1 DESCRIPTION
 
@@ -499,6 +576,8 @@ This distribution contains the following CLI utilities:
 
 =item * L<csv-add-field>
 
+=item * L<csv-avg>
+
 =item * L<csv-delete-field>
 
 =item * L<csv-list-field-names>
@@ -509,6 +588,8 @@ This distribution contains the following CLI utilities:
 
 =item * L<csv-sort-fields>
 
+=item * L<csv-sum>
+
 =back
 
 =head1 FUNCTIONS
@@ -555,6 +636,38 @@ that contains extra information.
 Return value:  (any)
 
 
+=head2 csv_avg(%args) -> [status, msg, result, meta]
+
+Output a summary row which are arithmetic averages of data rows.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<filename>* => I<filename>
+
+Input CSV file.
+
+=item * B<with_data_rows> => I<bool>
+
+Whether to also output data rows.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
 =head2 csv_delete_field(%args) -> [status, msg, result, meta]
 
 Delete a field from CSV file.
@@ -728,6 +841,38 @@ that contains extra information.
 Return value:  (any)
 
 
+=head2 csv_sum(%args) -> [status, msg, result, meta]
+
+Output a summary row which are arithmetic sums of data rows.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<filename>* => I<filename>
+
+Input CSV file.
+
+=item * B<with_data_rows> => I<bool>
+
+Whether to also output data rows.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
 =head2 csvutil(%args) -> [status, msg, result, meta]
 
 Perform action on a CSV file.
